//
//  GSBase.h
//  GlyphsCore
//
//  Created by Georg Seifert on 04.03.19.
//  Copyright © 2019 schriftgestaltung.de. All rights reserved.
//

#ifndef GSBase_h
#define GSBase_h
/**
  format of the .glyph file
 */
typedef NS_ENUM(uint8_t, GSFormatVersion) {
	/** the format in Glyphs 1 and 2*/
	GSFormatVersion1 = 1,
	/** the format in Glyphs 3 */
	GSFormatVersion3 = 3,
	/** the default format of the running app. Will change with when the format changes. Use this to always get the latest version */
	GSFormatVersionCurrent = 3,
};
/**
  if the file is stored as one flat file or as a bundle with one file per glyph.
 */
typedef NS_ENUM(uint8_t, GSPackageType) {
	GSPackageFlatFile = 1,
	GSPackageBundle = 2,
};

/** Used to determine how to gemerate the numbers features (pnum, lnum) */
typedef NS_ENUM(uint8_t, GSFigureStyles) {
	/** This is the initial state */
	NotTested = 0,
	/** There are no figures */
	NoFigures = 1,
	/** The default figures are tabular oldstyle figures */
	tosf = 2,
	/** The default figures are proportional oldstyle figures */
	osf = 3,
	/** The default figures are tabular figures */
	tf = 4,
	/** The default figures are lining figures */
	lf = 5
};

/**
 Possible writing directions
 */
typedef NS_ENUM(uint8_t, GSWritingDirection) {
	/**
	 a default value, not used
	 */
	GSWritingDirectionBIDI = 1,
	/**
	 Left to Right
	 */
	GSWritingDirectionLeftToRight = 0, // bit one and two not set
	/**
	 Right to Left
	 */
	GSWritingDirectionRightToLeft = 1 << 1,
	/**
	 Vertical
	 */
	GSWritingDirectionVertical = 1 << 2,
	/**
	 Line to Right
	 */
	GSWritingDirectionLineToRight = 1 << 3,
	
	GSKerningTypeContext = 1 << 7,
};

#define GSWritingDirectionIsLTR(d) ((d & GSWritingDirectionRightToLeft) == 0)
#define GSWritingDirectionIsRTL(d) ((d & GSWritingDirectionRightToLeft) == GSWritingDirectionRightToLeft)
#define GSWritingDirectionIsHorizontal(d) ((d & GSWritingDirectionVertical) == 0)
#define GSWritingDirectionIsVertical(d) ((d & GSWritingDirectionVertical) == GSWritingDirectionVertical)
#define GSWritingDirectionIsLineToRight(d) ((d & GSWritingDirectionLineToRight) == GSWritingDirectionLineToRight) && ((d & GSWritingDirectionVertical) == GSWritingDirectionVertical)
#define GSWritingDirectionIsLineToLeft(d) ((d & GSWritingDirectionLineToRight) == 0) && ((d & GSWritingDirectionVertical) == GSWritingDirectionVertical)

typedef NS_ENUM(int8_t, GSComponentAlignment) {
	GSAlignmentNoAligned = -2,
	GSAlignmentDisable = -1,
	GSAlignmentDefault = 0,
	GSAlignmentForce = 1,
	GSAlignmentAligned = 2,
	GSAlignmentHorizontal = 3,
};

typedef NS_ENUM(int8_t, GSElementOrientation) {
	GSAlignmentLeft = 0,
	GSAlignmentCenter = 2,
	GSAlignmentRight = 1,
	GSAlignmentNeutral = 4,
	GSAlignmentFit = 8
};

typedef NS_ENUM(uint8_t, GSCase) {
	GSNoCase = 0,
	GSUppercase = 1,
	GSLowercase = 2,
	GSSmallcaps = 3,
	GSMinor = 4,
	GSOtherCase = 5, // ??
};

typedef NS_ENUM(uint8_t, GSOutlineFormat) {
	GSOutlineFormatCFF = 1,
	GSOutlineFormatTrueType = 2,
	GSOutlineFormatVariableTT = 3,
	GSOutlineFormatVariableCFF = 4,
	GSOutlineFormatTTCubic = 5,
	GSOutlineFormatVariableTTCubic = 6,
};

NSString *GSNameForOutlineFormat(GSOutlineFormat outlineFormat);
NSString *GSFileExtensionForOutlineFormat(GSOutlineFormat outlineFormat);

extern NSString * const GSFontFormatOTFName;
extern NSString * const GSFontFormatTTFName;
extern NSString * const GSFontFormatTTCubicName;
extern NSString * const GSFontFormatWOFFName;
extern NSString * const GSFontFormatWOFF2Name;
extern NSString * const GSFontFormatEOTName;


#endif /* GSBase_h */
